#if !defined(AVIFILE_H)
#define AVIFILE_H

#pragma comment(lib,"vfw32")
#pragma warning(disable:4018)

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MAX_AUDIO_STREAMS 5
#define MAX_VIDEO_STREAMS 5

#include <Windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <vfw.h>

class CAviFile  
{
public:
	CAviFile()
	{
		AVIFileInit();
		Init();
	};
	virtual ~CAviFile()
	{
		AVIFileExit();
		Release();
	};
	void Release()
	{
		if(m_pAviFile)
			AVIFileRelease(m_pAviFile);
		
		int n;
		for(n = 0; n < m_nNumAudioStreams; n++)
		{
			if(m_pAudioStreams[n])
				AVIStreamRelease(m_pAudioStreams[n]);

			if(m_pAudioFormats[n])
				delete [] ((LPBYTE)m_pAudioFormats[n]);

			if(m_pAudioData[n])
				delete [] m_pAudioData[n];
		}

		for(n = 0; n < m_nNumVideoStreams; n++)
		{
			if(m_pVideoStreams[n])
				AVIStreamRelease(m_pVideoStreams[n]);

			if(m_pVideoFormats[n])
				delete [] ((LPBYTE)m_pVideoFormats[n]);
		}

		Init();
	};

	bool Open(LPCSTR lpszFileName)
	{
		if(AVIFileOpen(&m_pAviFile, lpszFileName, OF_READ, NULL))
			return false;

		FindStreams();
		
		if(!DetermineAudioFormats() ||
			!DetermineVideoFormats())
		{
			Release();
			return false;
		}

		return true;
	};

	int GetAudioStreamCount()
	{
		return m_nNumAudioStreams;
	};
	int GetVideoStreamCount()
	{
		return m_nNumVideoStreams;
	};
	LPWAVEFORMAT GetAudioFormat(UINT nStreamNum)
	{
		if(nStreamNum >= m_nNumAudioStreams)
			return NULL;
		return m_pAudioFormats[nStreamNum];
	};
	LPBITMAPINFO GetVideoFormat(UINT nStreamNum)
	{
		if(nStreamNum >= m_nNumVideoStreams)
			return NULL;

		return m_pVideoFormats[nStreamNum];
	};

	LPBYTE ExtractAudioStream(UINT nStreamNum)
	{
		if(m_pAudioData[nStreamNum])
			return m_pAudioData[nStreamNum];
		
		if(nStreamNum >= m_nNumAudioStreams)
			return NULL;

		PAVISTREAM pStream = m_pAudioStreams[nStreamNum];

		LONG lSize;
		if(AVIStreamRead(pStream, 0, AVISTREAMREAD_CONVENIENT, NULL, 0, &lSize, NULL))
			return NULL;

		LPBYTE pBuffer = new BYTE[lSize];
		if(!pBuffer)
			return NULL;

		if(AVIStreamRead(pStream, 0, AVISTREAMREAD_CONVENIENT, pBuffer, lSize, NULL, NULL))
			return NULL;

		m_pAudioData[nStreamNum] = pBuffer;

		return pBuffer;
	};

	bool StartVideoRetrieve(UINT nStreamNum)
	{
		if(nStreamNum >= m_nNumVideoStreams)
			return false;

		PAVISTREAM pStream = m_pVideoStreams[nStreamNum];

		PGETFRAME &pgf = m_pVideoPGF[nStreamNum];
		
		pgf = AVIStreamGetFrameOpen(pStream, NULL/*lpDesiredFormat*/);

		if(!pgf)
			return false;

		m_lVideoEndTime[nStreamNum] = AVIStreamEndTime(pStream);

		return true;
	};

	// In these functions:
	//	NULL in last param means no more frames.
	//  Frame is only valid until the next
	//  call of one of these functions, or EndVideoRetrieve.
	void GetVideoFrameAtTime(UINT nStreamNum, LONG lTimeInMilliSec, LPBITMAPINFOHEADER *ppbi)
	{
		if(nStreamNum >= m_nNumVideoStreams)
		{
			*ppbi = NULL;
			return;
		}

		PAVISTREAM pStream = m_pVideoStreams[nStreamNum];
		PGETFRAME &pgf = m_pVideoPGF[nStreamNum];

		LONG lFrame;
		if(lTimeInMilliSec <= m_lVideoEndTime[nStreamNum])
			lFrame = AVIStreamTimeToSample(pStream, lTimeInMilliSec);
		else
		{
			*ppbi = NULL; // video is done, no more frames
			return;
		}

		*ppbi = (LPBITMAPINFOHEADER)AVIStreamGetFrame(pgf, lFrame);
	};
	void GetVideoFrame(UINT nStreamNum, LONG lFrame, LPBITMAPINFOHEADER *ppbi)
	{
		PGETFRAME &pgf = m_pVideoPGF[nStreamNum];
		*ppbi = (LPBITMAPINFOHEADER)AVIStreamGetFrame(pgf, lFrame);
		// NULL on error or no more frames
	};

	bool EndVideoRetrieve(UINT nStreamNum)
	{
		PGETFRAME &pgf = m_pVideoPGF[nStreamNum];

		if(AVIStreamGetFrameClose(pgf))
			return false;

		pgf = NULL;

		return true;
	};
	
protected:
	void Init()
	{
		m_pAviFile = NULL;
		m_nNumAudioStreams = 0;
		m_nNumVideoStreams = 0;

		int n;
		
		for(n = 0; n < MAX_AUDIO_STREAMS; n++)
		{
			m_pAudioStreams[n] = NULL;
			m_pAudioFormats[n] = NULL;
			m_pAudioData[n] = NULL;
		}

		for(n = 0; n < MAX_VIDEO_STREAMS; n++)
		{
			m_pVideoStreams[n] = NULL;
			m_pVideoFormats[n] = NULL;
			m_pVideoPGF[n] = NULL;
			m_lVideoEndTime[n] = 0;
		}
	};
	void FindStreams()
	{
		do
		{
			if(AVIFileGetStream(m_pAviFile, &m_pAudioStreams[m_nNumAudioStreams],
				streamtypeAUDIO, m_nNumAudioStreams))
				break;
		}
		while(++m_nNumAudioStreams < MAX_AUDIO_STREAMS);
		
		do
		{
			if(AVIFileGetStream(m_pAviFile, &m_pVideoStreams[m_nNumVideoStreams],
				streamtypeVIDEO, m_nNumVideoStreams))
				break;
		}
		while(++m_nNumVideoStreams < MAX_VIDEO_STREAMS);
	};


	bool DetermineAudioFormats()
	{
		for(int n = 0; n < m_nNumAudioStreams; n++)
		{
			PAVISTREAM pStream = m_pAudioStreams[n];

			LONG lSize;
			if(AVIStreamReadFormat(pStream, AVIStreamStart(pStream), NULL, &lSize))
				return false;

			LPBYTE pChunk = new BYTE[lSize];
			if(!pChunk)
				return false;

			if(AVIStreamReadFormat(pStream, AVIStreamStart(pStream), pChunk, &lSize))
				return false;

			m_pAudioFormats[n] = (LPWAVEFORMAT)pChunk;
		}

		return true;
	};


	bool DetermineVideoFormats()
	{
		for(int n = 0; n < m_nNumVideoStreams; n++)
		{
			PAVISTREAM pStream = m_pVideoStreams[n];

			LONG lSize;
			if(AVIStreamReadFormat(pStream, AVIStreamStart(pStream), NULL, &lSize))
				return false;

			LPBYTE pChunk = new BYTE[lSize];
			if(!pChunk)
				return false;

			if(AVIStreamReadFormat(pStream, AVIStreamStart(pStream), pChunk, &lSize))
				return false;

			m_pVideoFormats[n] = (LPBITMAPINFO)pChunk;
		}
		return true;
	};

	PAVIFILE m_pAviFile;

	PAVISTREAM
		m_pAudioStreams[MAX_AUDIO_STREAMS],
		m_pVideoStreams[MAX_VIDEO_STREAMS];

	int m_nNumAudioStreams,
		m_nNumVideoStreams;

	LPWAVEFORMAT m_pAudioFormats[MAX_AUDIO_STREAMS];
	LPBYTE m_pAudioData[MAX_AUDIO_STREAMS];

	LPBITMAPINFO m_pVideoFormats[MAX_VIDEO_STREAMS];
	PGETFRAME m_pVideoPGF[MAX_VIDEO_STREAMS];
	LONG m_lVideoEndTime[MAX_VIDEO_STREAMS];
};

#endif
