//------------------------------------------------------------------------------
// File: Timer.h
//
// Copyright (c) 2001, Alexandre Bernardino.  All rights reserved.
//------------------------------------------------------------------------------


#ifndef _timer_h_
#define _timer_h_

#include <math.h>
#include <float.h>


class CTimer
{	
private:
	__int64 m_i64CPUFrequency;
	LARGE_INTEGER ClockStart, ClockRead, CounterStart, CounterLap, CounterRead;
	CRITICAL_SECTION cs;
protected:	
	long  m_lCycles;
	float m_fTotalCycleTime;
	float m_fTotalProcessingTime;
	float m_fCurrentCycleTime;
	float m_fCurrentProcessingTime;
	float m_fCurrentSamplingRate;
	float m_fDecayAverageProcessingTime;
	float m_fPreviousDecayAverageProcessingTime;
	float m_fDecayAverageSamplingRate;
	float m_fPreviousDecayAverageSamplingRate;
	float m_fDecayAverageRate;
	float m_fMaxSamplingRate;
	float m_fMaxProcessingTime;
	float m_fMinSamplingRate;
	float m_fMinProcessingTime;
public:
	CTimer() :	m_lCycles(0),
				m_fTotalCycleTime(0.0f),
				m_fTotalProcessingTime(0.0f),
				m_fCurrentCycleTime(0.0f),
				m_fCurrentProcessingTime(0.0f),
				m_fCurrentSamplingRate(0.0f),
				m_fDecayAverageProcessingTime(0.0f),
				m_fPreviousDecayAverageProcessingTime(0.0f),
				m_fDecayAverageSamplingRate(0.0f),
				m_fPreviousDecayAverageSamplingRate(0.0f)
	{		
		LARGE_INTEGER li;
		QueryPerformanceFrequency(&li);
		m_i64CPUFrequency = li.QuadPart;
		m_fMaxSamplingRate = FLT_MIN;
		m_fMinSamplingRate = FLT_MAX;
		m_fMaxProcessingTime = FLT_MIN;
		m_fMinProcessingTime = FLT_MAX;
		InitializeCriticalSection(&cs);
	}

	
	STDMETHODIMP StartClock()		
	{		
		QueryPerformanceCounter(&ClockStart);
		return S_OK;
	}

	STDMETHODIMP ReadClock( float *time )
	{
		QueryPerformanceCounter(&ClockRead);
		*time = static_cast<float>(ClockRead.QuadPart - ClockStart.QuadPart)/static_cast<float>(m_i64CPUFrequency);
		return S_OK;
	}


	STDMETHODIMP StartTimer()		
	{		
		EnterCriticalSection(&cs);

			m_lCycles = 0;
			m_fTotalCycleTime = 0.0f;
			m_fTotalProcessingTime = 0.0f;
			m_fCurrentCycleTime = 0.0f;
			m_fCurrentProcessingTime = 0.0f;
			m_fCurrentSamplingRate = 0.0f;
			m_fDecayAverageProcessingTime = 0.0f;
			m_fPreviousDecayAverageProcessingTime = 0.0f;
			m_fDecayAverageSamplingRate = 0.0f;
			m_fPreviousDecayAverageSamplingRate = 0.0f;
			m_fMaxSamplingRate = FLT_MIN;
			m_fMinSamplingRate = FLT_MAX;
			m_fMaxProcessingTime = FLT_MIN;
			m_fMinProcessingTime = FLT_MAX;
			QueryPerformanceCounter(&CounterStart);
			CounterLap = CounterStart;
			CounterRead = CounterStart;

		LeaveCriticalSection(&cs);
		return S_OK;
	};
	

	STDMETHODIMP MarkSectionStart() 
	{
		QueryPerformanceCounter(&CounterLap);
		return S_OK;
	};

	STDMETHODIMP MarkSectionEnd() 
	{
		EnterCriticalSection(&cs);			

			//readings for sampling rate
			float elapsedTime;
			QueryPerformanceCounter(&CounterRead);
			elapsedTime = static_cast<float>(CounterRead.QuadPart - CounterStart.QuadPart)/static_cast<float>(m_i64CPUFrequency);
			m_fCurrentCycleTime =  elapsedTime - m_fTotalCycleTime;
			assert(m_fCurrentCycleTime > 0);
			m_fTotalCycleTime = elapsedTime;
			assert(m_fCurrentCycleTime != 0.0f);
			m_fCurrentSamplingRate = 1.0f/m_fCurrentCycleTime;
			if(m_fCurrentSamplingRate < m_fMinSamplingRate)
				m_fMinSamplingRate = m_fCurrentSamplingRate;
			if(m_fCurrentSamplingRate > m_fMaxSamplingRate)
				m_fMaxSamplingRate = m_fCurrentSamplingRate;


			//readings for processing time
			m_fCurrentProcessingTime = static_cast<float>(CounterRead.QuadPart - CounterLap.QuadPart)/static_cast<float>(m_i64CPUFrequency);
			m_fTotalProcessingTime += m_fCurrentProcessingTime;
			if(m_fCurrentProcessingTime < m_fMinProcessingTime)
				m_fMinProcessingTime = m_fCurrentProcessingTime;
			if(m_fCurrentProcessingTime > m_fMaxProcessingTime)
				m_fMaxProcessingTime = m_fCurrentProcessingTime;


			// running averages
			m_fPreviousDecayAverageProcessingTime = m_fDecayAverageProcessingTime;
			m_fPreviousDecayAverageSamplingRate = m_fDecayAverageSamplingRate;

			// configuring decay rate in order to get a settling time of about 1 sec.
			if(m_fCurrentCycleTime >= 1.0f)
				m_fDecayAverageRate = 0.95f;
			else
				m_fDecayAverageRate = static_cast<float>(1.0 - pow(0.05, m_fCurrentCycleTime));

			assert(m_fDecayAverageRate > 0);
			assert(m_fDecayAverageRate < 1);

			m_fDecayAverageProcessingTime = (1 - m_fDecayAverageRate) * m_fPreviousDecayAverageProcessingTime +
											m_fDecayAverageRate * m_fCurrentProcessingTime;
			m_fDecayAverageSamplingRate =	(1 - m_fDecayAverageRate) * m_fPreviousDecayAverageSamplingRate  +
											m_fDecayAverageRate * m_fCurrentSamplingRate;

			InterlockedIncrement(&m_lCycles);
		LeaveCriticalSection(&cs);
		return S_OK;
	};

	
	STDMETHODIMP GetAvgProcessingTime(float *val) 
	{
		EnterCriticalSection(&cs);

			if( 0 != m_lCycles )
				*val = m_fTotalProcessingTime/m_lCycles;	
			else
				*val = 0;

		LeaveCriticalSection(&cs);
		return S_OK;
	};
	STDMETHODIMP GetAvgSamplingRate(float *val) 
	{
		EnterCriticalSection(&cs);

			if( 0 != m_lCycles )
				*val = m_lCycles/m_fTotalCycleTime;	
			else
				*val = 0;

		LeaveCriticalSection(&cs);
		return S_OK;
	};
	STDMETHODIMP GetDecayAvgProcessingTime(float *val) 
	{
		*val = m_fDecayAverageProcessingTime;
		return S_OK;
	};
	STDMETHODIMP GetDecayAvgSamplingRate(float *val) 
	{
		*val = m_fDecayAverageSamplingRate;
		return S_OK;
	};
	STDMETHODIMP GetMaxProcessingTime(float *val) 
	{
		if( 0 != m_lCycles )
			*val = m_fMaxProcessingTime;	
		else
			*val = 0;
		return S_OK;
	};
	STDMETHODIMP GetMaxSamplingRate(float *val) 
	{
		if( 0 != m_lCycles )
			*val = m_fMaxSamplingRate;	
		else
			*val = 0;
		return S_OK;
	};
	STDMETHODIMP GetMinProcessingTime(float *val) 
	{
		if( 0 != m_lCycles )
			*val = m_fMinProcessingTime;	
		else
			*val = 0;
		return S_OK;
	};
	STDMETHODIMP GetMinSamplingRate(float *val) 
	{
		if( 0 != m_lCycles )
			*val = m_fMinSamplingRate;	
		else
			*val = 0;
		return S_OK;
	};
	STDMETHODIMP GetProcessingTime(float *val) 
	{
		if( 0 != m_lCycles )
			*val = m_fCurrentProcessingTime;	
		else
			*val = 0;
		return S_OK;
	};
	STDMETHODIMP GetSamplingRate(float *val) 
	{
		if( 0 != m_lCycles )
			*val = m_fCurrentSamplingRate;	
		else
			*val = 0;
		return S_OK;
	};
};

#endif